import logging
from bluemax.scripts.pid_script import Stoppable


logger = logging.getLogger(__name__)


class Server(Stoppable):
    def start(self):
        """
            Called as console_script to run the
            server - either standalone or as manager
            of workers using redis.
        """
        from bluemax.web.settings import load_config, options
        from bluemax.web import server
        load_config('.env')
        logger.info('mode:%s', 'debug' if options.debug else 'prod')
        server.main()

def main():
    Server().main()

if __name__ == '__main__':
    main()



import tornado.web
import tornado.auth
from tornado import escape
from tornado.options import options
from .authentication import LoginHandler, UserMixin
from urllib.parse import urlencode
from ...utils import json_utils
import logging


class ProfileHandler(UserMixin, tornado.web.RequestHandler):

    @tornado.web.authenticated
    async def get(self):
        self.finish(json_utils.dumps(self.current_user, indent=4))


class MSLoginHandler(LoginHandler,
                     tornado.auth.OAuth2Mixin):

    _USER_URL = "https://graph.microsoft.com/v1.0/me"
    _OAUTH_AUTHORIZE_URL = "https://login.microsoftonline.com/common/oauth2/v2.0/authorize"
    _OAUTH_ACCESS_TOKEN_URL = "https://login.microsoftonline.com/common/oauth2/v2.0/token"

    async def get(self):
        if self.get_argument('code', False):
            access = await self.get_authenticated_user(
                redirect_uri=self.settings["login_url"],
                client_id=self.settings['ms_oauth']['key'],
                client_secret=self.settings["ms_oauth"]["secret"],
                code=self.get_argument('code'),
                extra_params={
                    "scope": 'User.Read profile openid email'
                })
            user = await self.get_user_profile(
                self._USER_URL,
                access_token=access["access_token"])
            logging.info(user)
            self.set_current_user(user)
            self.redirect(self.get_argument("state"))
        else:
            await self.authorize_redirect(
                redirect_uri=self.settings["login_url"],
                client_id=self.settings['ms_oauth']['key'],
                client_secret=self.settings["ms_oauth"]["secret"],
                scope=['User.Read'],
                response_type='code',
                extra_params={
                    'approval_prompt': 'auto',
                    'state': self.get_argument("next")
                })


    async def get_user_profile(self, url, access_token):
        http = self.get_auth_http_client()
        try:
            response = await http.fetch(url,headers={'Authorization': access_token})
        except Exception as ex:
            logging.exception(ex)
            raise
        logging.debug(response.body)
        return escape.json_decode(response.body)


    async def get_authenticated_user(self, redirect_uri, client_id, client_secret,
                                     code, extra_params=None):
        http = self.get_auth_http_client()
        args = {
            "redirect_uri": redirect_uri,
            "code": code,
            "client_id": client_id,
            "client_secret": client_secret,
            "grant_type": "authorization_code"
        }
        if extra_params:
                args.update(extra_params)
        body = urlencode(args)
        try:
            response = await http.fetch(self._OAUTH_ACCESS_TOKEN_URL,
                                        method="POST",
                                        headers={'Content-Type': 'application/x-www-form-urlencoded'},
                                        body=body)
        except Exception as ex:
            logging.info(body)
            logging.exception(ex)
            raise
        logging.debug(response.body)
        args = escape.json_decode(response.body)
        session = {
            "access_token": args.get("access_token"),
            "expires_in": args.get("expires_in")
        }
        return session



from bluemax import context
import time
import asyncio
from tornado import gen
from typing import Tuple


__all__ = ["add", "a_add", "t_add"]


def add(a:int=2, b:int=2, sleep_for:float=0.1) -> Tuple[str,int]:
    """ returns a plus b syncronous."""
    result = a+b
    time.sleep(sleep_for)
    context.broadcast_on_success("action", {
        "name": "add",
        "result": result,
        "user": context.get_current_user() })
    return "add", result


async def a_add(a:int=2, b:int=3, sleep_for:float=0.2) -> Tuple[str,int]:
    """ returns a plus b, async native."""
    result = a+b
    await asyncio.sleep(sleep_for)
    context.broadcast_on_success("action", {
        "name": "a_add",
        "result": result,
        "user": context.get_current_user() })
    return "a_add", result


@gen.coroutine
def t_add(a:int=2, b:int=4, sleep_for:float=0.3) -> Tuple[str,int]:
    """ returns a plus b, async tornado."""
    result = a+b
    yield gen.sleep(sleep_for)
    context.broadcast_on_success("action", {
        "name": "t_add",
        "result": result,
        "user": context.get_current_user() })
    return "t_add", result



from bluemax.utils.config import load_config
from tornado.options import define, options

define("debug",type=bool,default=True,help="auto reload etc")
define("port",type=int,default=8080,help="port to listen on")
define("workers",type=int,default=1,help="number of worker threads")
define("procedures",type=str,default="",help="procedures to make available")
define("static_dir",type=str,default=None,help="static web directory")
define("redis_url", default="", help="redis url")
define("redis_work_q", default="_work_q_", help="name of queue to rpush to")

define("cookie_name", type=str, default="bluemax", help="bluemax session cooklie name")
define("cookie_secret", type=str, default="it was a dark and stormy night, blue max", help="session secret")
define("login_url",type=str,default='/login',help="url for login")
define("auth_class",type=str,default=None,help="class of login handlers")

define("ms_app_id", type=str, help="MS App ID for authentication")
define("ms_app_secret", type=str, help="MS App Secret for authentication")

define("settings_extend", type=str, help="fully qualified function to import and call")
define("urls_extend", type=str, help="fully qualified function to import and call")


def settings():
    result = {
        "cookie_secret": options.cookie_secret,
        "login_url": options.login_url,
        "debug": options.debug
    }
    return result



import logging
from bluemax.scripts.pid_script import Stoppable


logger = logging.getLogger(__name__)


class Worker(Stoppable):
    def start(self):
        """
            Called as console_script to run the
            worker using redis.
        """
        from bluemax.web.settings import load_config, options
        from bluemax.work import worker
        load_config('.env')
        logger.info('hello from worker! mode:%s', 'debug' if options.debug else 'prod')
        worker.main()

def main():
    Worker().main()


if __name__ == "__main__":
    main()



from .perform import get_current_user, broadcast, broadcast_on_success, \
    put_work, register_object, deregister_object

__all__ = [
    "get_current_user",
    "broadcast",
    "broadcast_on_success",
    "put_work",
    "register_object",
    "deregister_object",
    ]



from ldap3 import Server, Connection, SIMPLE, SYNC, ASYNC, SUBTREE, ALL
from ldap3.core.exceptions import LDAPBindError
from tornado.options import options
from ...utils import json_utils
from .authentication import LoginHandler
import logging

AUTH_LDAP_ATTRS = [
    'cn',
    'cn',
    'department',
    'description',
    'displayname',
    'distinguishedname',
    'givenname', # firstname
    'mail',
    'memberof',
    'name', # full name
    'objectclass',
    'samaccountname',
    'sn', # last name or surname
    'telephonenumber',
    'title',
    'useraccountcontrol',
    'userprincipalname',
    'whencreated',
]
AUTH_LDAP_SUPERUSERS = [
    'pshingavi@directbuy.com',
    'sashford@directbuy.com',
    'patrick.lowe@directbuy.com',
    'fleng@directbuy.com',
    'peter.bunyan@directbuy.com',
]


def _login_(username, password):
    try:
        s = Server(options.LDAP_HOST, get_info=ALL)
        c = Connection(s,
                       auto_bind = True,
                       client_strategy = SYNC,
                       user=username.strip(),
                       password=password.strip(),
                       authentication=SIMPLE,
                       check_names=True)
        filter_str = f'(&(!(useraccountcontrol=514))(userprincipalname={c.user}))'
        logging.info(filter_str)
        r = c.search(search_base='dc=ucctops,dc=com',
                     search_filter=filter_str,
                     search_scope=SUBTREE,
                     attributes=AUTH_LDAP_ATTRS)
        if r is True:
            user = json_utils.loads(c.entries[0].entry_to_json())
            result = {
                "username": username,
                "user": user
            }
            logging.info("logged in %r", result)
            return result
        result = {
            "username": username
        }
        logging.info("logged in %r", result)
        return result
    except LDAPBindError as ex:
       logging.info("login failed: %s", ex)


class LDAP3LoginHandler(LoginHandler):

    def login(self, username, password):
        if username in AUTH_LDAP_SUPERUSERS:
            return _login_(username, password)



from setuptools import setup, find_packages
try:
    from wheel.bdist_wheel import bdist_wheel as _bdist_wheel
    class bdist_wheel(_bdist_wheel):
        def finalize_options(self):
            _bdist_wheel.finalize_options(self)
            self.root_is_pure = True
except ImportError:
    bdist_wheel = None


with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name='bluemax',
    version='0.0.40',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/blueshed/bluemax/",
    packages=find_packages(exclude=['tests.*','tests']),
    package_data={'bluemax.web': ['static/*']},
    include_package_data = True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'Click',
        'tornado',
    ],
    extras_require={
        'sa':  ["alembic","sqlalchemy"],
        'redis': ["aioredis","redis"],
        'all': ["alembic","sqlalchemy","aioredis","redis"],
        'dev': ["pytest-tornado","wheel","twine"]
    },
    entry_points={
        'console_scripts': [
            "runmax=bluemax.scripts.runmax:cli",
        ]
    },
)
